@@ -58,6 +58,8 @@ except ImportError:
 else:
     HAVE_DOCUTILS = True
 
+VERSION = "0.9.0"
+
 class Config:
     """
     @ivar files_read: list of config files that were parsed.
@@ -76,10 +78,16 @@ class Config:
                     'appdir' : app_dir,
                     'home' : home,
                     'housekeeping_interval' : '2',
+                    'case_sensitive' : '0',
+                    'search_tags' : '1',
                     'sort_mode' : '1',
+                    'pinned_ontop' : '1',
                     'db_path' : os.path.join(home, '.nvpy'),
-                    'font_family' : 'Courier',
+                    'font_family' : 'Courier', # monospaced on all platforms
                     'font_size' : '12',
+                    'list_font_family' : 'Helvetica', # sans on all platforms
+                    'list_font_size' : '10',
+                    'background_color' : 'white',
                     'sn_username' : '',
                     'sn_password' : ''
                    }
@@ -110,13 +118,22 @@ class Config:
         # make logic to find in $HOME if not set
         self.db_path = cp.get(cfg_sec, 'db_path')
         #  0 = alpha sort, 1 = last modified first
+        self.case_sensitive = cp.getint(cfg_sec, 'case_sensitive')
+        self.search_tags = cp.getint(cfg_sec, 'search_tags')
         self.sort_mode = cp.getint(cfg_sec, 'sort_mode')
+        self.pinned_ontop = cp.getint(cfg_sec, 'pinned_ontop')
         self.housekeeping_interval = cp.getint(cfg_sec, 'housekeeping_interval')
         self.housekeeping_interval_ms = self.housekeeping_interval * 1000
         
         self.font_family = cp.get(cfg_sec, 'font_family')
         self.font_size = cp.getint(cfg_sec, 'font_size')
-        
+
+        self.list_font_family = cp.get(cfg_sec, 'list_font_family')
+        self.list_font_size = cp.getint(cfg_sec, 'list_font_size')
+
+        self.background_color = cp.get(cfg_sec, 'background_color')
+
+
 class NotesListModel(SubjectMixin):
     """
     @ivar list: List of (str key, dict note) objects.
@@ -162,7 +179,7 @@ class Controller:
         
         # should probably also look in $HOME
         self.config = Config(self.appdir)
-        self.config.app_version = self.get_version()
+        self.config.app_version = VERSION
 
         # configure logging module
         #############################
@@ -188,7 +205,7 @@ class Controller:
         # read our database of notes into memory
         # and sync with simplenote.
         c = self.config
-        notes_db_config = KeyValueObject(db_path=c.db_path, sn_username=c.sn_username, sn_password=c.sn_password, sort_mode=c.sort_mode)
+        notes_db_config = KeyValueObject(db_path=c.db_path, sn_username=c.sn_username, sn_password=c.sn_password, sort_mode=c.sort_mode, pinned_ontop=c.pinned_ontop, case_sensitive=c.case_sensitive, search_tags=c.search_tags)
         self.notes_db = NotesDB(notes_db_config)
         self.notes_db.add_observer('synced:note', self.observer_notes_db_synced_note)
         self.notes_db.add_observer('change:note-status', self.observer_notes_db_change_note_status)
@@ -205,6 +222,8 @@ class Controller:
         self.view.add_observer('select:note', self.observer_view_select_note)
         self.view.add_observer('change:entry', self.observer_view_change_entry)
         self.view.add_observer('change:text', self.observer_view_change_text)
+        self.view.add_observer('change:tags', self.observer_view_change_tags)
+        self.view.add_observer('change:pinned', self.observer_view_change_pinned)
         self.view.add_observer('create:note', self.observer_view_create_note)
         self.view.add_observer('keep:house', self.observer_view_keep_house)
         self.view.add_observer('command:markdown',
@@ -226,19 +245,16 @@ class Controller:
         # we only use idx, because key could change from right under us.
         self.selected_note_idx = -1
         self.view.select_note(0)
-        
+
         # perform full sync with server, and refresh notes list if successful
         self.sync_full()
-                
+
     def get_selected_note_key(self):
         if self.selected_note_idx >= 0:
             return self.notes_list_model.list[self.selected_note_idx].key
         else:
             return None
                 
-    def get_version(self):
-        return "0.8.5"
-    
     def main_loop(self):
         if not self.config.files_read:
             self.view.show_warning('No config file', 
@@ -258,7 +274,7 @@ class Controller:
             self.view.set_note_status(self.notes_db.get_note_status(skey))
             
     def observer_notes_db_sync_full(self, notes_db, evt_type, evt):
-	logging.debug(evt.msg)
+        logging.debug(evt.msg)
         self.view.set_status_text(evt.msg)
         
     def observer_notes_db_synced_note(self, notes_db, evt_type, evt):
@@ -272,11 +288,11 @@ class Controller:
         
         if selected_note_o.key == evt.lkey:
             if selected_note_o.note['content'] != evt.old_note['content']:
-                self.view.mute('change:text')
+                self.view.mute_note_data_changes()
                 # in this case, we want to keep the user's undo buffer so that they
                 # can undo synced back changes if they would want to.
-                self.view.set_text(selected_note_o.note['content'], reset_undo=False)
-                self.view.unmute('change:text')
+                self.view.set_note_data(selected_note_o.note, reset_undo=False)
+                self.view.unmute_note_data_changes()
 
     def observer_view_click_notelink(self, view, evt_type, note_name):
         # find note_name in titles, try to jump to that note
@@ -298,13 +314,23 @@ class Controller:
         # if these two are not equal, something is not kosher.
         assert(evt.sel == self.selected_note_idx)
 
-        # delete the note        
+        # first get key of note that is to be deleted
         key = self.get_selected_note_key()
+
+        # then try to select after the one that is to be deleted
+        nidx = evt.sel + 1
+        if nidx >= 0 and nidx < self.view.get_number_of_notes():
+            self.view.select_note(nidx)
+
+        # finally delete the note
         self.notes_db.delete_note(key)
         
         # easiest now is just to regenerate the list by resetting search string
+        # if the note after the deleted one is already selected, this will
+        # simply keep that selection!
         self.view.set_search_entry_text(self.view.get_search_entry_text())
 
+
     def helper_markdown_to_html(self):
         if self.selected_note_idx >= 0:
             key = self.notes_list_model.list[self.selected_note_idx].key
@@ -412,8 +438,8 @@ class Controller:
             # this call will update our in-memory version if necessary
             ret = self.notes_db.sync_note_unthreaded(key)
             if ret and ret[1] == True:
-                self.view.update_selected_note_text(
-                        self.notes_db.notes[key]['content'])
+                self.view.update_selected_note_data(
+                        self.notes_db.notes[key])
                 self.view.set_status_text(
                 'Synced updated note from server.')
 
@@ -462,7 +488,21 @@ class Controller:
             key = self.notes_list_model.list[self.selected_note_idx].key
             self.notes_db.set_note_content(key,
                                            self.view.get_text())
-            
+
+    def observer_view_change_tags(self, view, evt_type, evt):
+        # get new text and update our database
+        # need local key of currently selected note for this
+        if self.selected_note_idx >= 0:
+            key = self.notes_list_model.list[self.selected_note_idx].key
+            self.notes_db.set_note_tags(key, evt.value)
+
+    def observer_view_change_pinned(self, view, evt_type, evt):
+        # get new text and update our database
+        # need local key of currently selected note for this
+        if self.selected_note_idx >= 0:
+            key = self.notes_list_model.list[self.selected_note_idx].key
+            self.notes_db.set_note_pinned(key, evt.value)
+
     def observer_view_close(self, view, evt_type, evt):
         # check that everything has been saved and synced before exiting
         
@@ -498,33 +538,39 @@ class Controller:
     def select_note(self, idx):
         if idx >= 0:
             key = self.notes_list_model.list[idx].key
-            c = self.notes_db.get_note_content(key)
+            note = self.notes_db.get_note(key)
 
         else:
             key = None
-            c = ''
+            note = None
             idx = -1
         
         self.selected_note_idx = idx
 
-        # when we do this, we don't want the change:text event thanks
-        self.view.mute('change:text')
-        self.view.set_text(c)
+        # when we do this, we don't want the change:{text,tags,pinned} events
+        # because those should only fire when they are changed through the UI
+        self.view.mute_note_data_changes()
+        self.view.set_note_data(note)
         if key:
             self.view.set_note_status(self.notes_db.get_note_status(key))
-        self.view.unmute('change:text')
+
+        self.view.unmute_note_data_changes()
 
     def sync_full(self):
         try:
-            self.notes_db.sync_full()
-        except SyncError:
-            pass
+            sync_from_server_errors = self.notes_db.sync_full()
+
+        except SyncError as e:
+            self.view.show_error('Sync error', e.message)
         
         else:
             # regenerate display list
             # reselect old selection
             # put cursor where it used to be.
             self.view.refresh_notes_list()
+
+            if sync_from_server_errors > 0:
+                self.view.show_error('Error syncing notes from server', 'Error syncing %d notes from server. Please check nvpy.log for details.' % (sync_from_server_errors,))
         
 
 def main():
