@@ -14,6 +14,8 @@ import tkMessageBox
 import utils
 import webbrowser
 
+from datetime import datetime
+
 class WidgetRedirector:
 
     """Support for redirecting arbitrary widget subcommands."""
@@ -145,6 +147,338 @@ class StatusBar(tk.Frame):
         self.status.config(text="")
         self.status.update_idletasks()
 
+class NotesList(tk.Frame):
+    """
+    @ivar note_headers: list containing tuples with each note's title, tags,
+    modified date and so forth. Always in sync with what is displayed.
+    """
+
+    TITLE_COL = 0
+    TAGS_COL = 1
+    MODIFYDATE_COL = 2
+    PINNED_COL = 3
+
+    def __init__(self, master, font_family, font_size, background_color):
+        tk.Frame.__init__(self, master)
+
+        yscrollbar = tk.Scrollbar(self)
+        yscrollbar.pack(side=tk.RIGHT, fill=tk.Y)
+
+        f = tkFont.Font(family=font_family, size=font_size)
+        # tkFont.families(root) returns list of available font family names
+        # this determines the width of the complete interface (yes)
+        # size=-self.config.font_size
+        self.text = tk.Text(self, height=25, width=30,
+            wrap=tk.NONE,
+            font=f,
+            yscrollcommand=yscrollbar.set,
+            undo=True,
+            background = background_color)
+        # change default font at runtime with:
+        #text.config(font=f)
+
+        self.text.config(cursor="arrow")
+        self.disable_text()
+        self.text.pack(fill=tk.BOTH, expand=1)
+
+        # tags for all kinds of styling ############################
+        ############################################################
+
+        self.text.tag_config("selected", background="light blue")
+
+        self.text.tag_config("pinned", foreground="dark gray")
+
+        # next two lines from:
+        # http://stackoverflow.com/a/9901862/532513
+        bold_font = tkFont.Font(self.text, self.text.cget("font"))
+        bold_font.configure(weight="bold")
+        self.text.tag_config("title", font=bold_font)
+
+        italic_font = tkFont.Font(self.text, self.text.cget("font"))
+        italic_font.configure(slant="italic")
+        self.text.tag_config("tags", font=italic_font, foreground="dark gray")
+        self.text.tag_config("found", font=italic_font, foreground="dark gray", background="lightyellow")
+
+        self.text.tag_config("modifydate", foreground="dark gray")
+
+        yscrollbar.config(command=self.text.yview)
+
+        self._bind_events()
+
+        self.selected_idx = -1
+        # list containing tuples with each note's title, tags,
+        self.note_headers = []
+
+    def append(self, note, config):
+        """
+        @param note: The complete note dictionary.
+        """
+
+
+        title = utils.get_note_title(note)
+        tags = note.get('tags')
+        modifydate = float(note.get('modifydate'))
+        pinned = utils.note_pinned(note)
+        self.note_headers.append((title, tags, modifydate, pinned))
+
+        self.enable_text()
+
+
+        self.text.insert(tk.END, title, ("title,"))
+
+        if pinned:
+            self.text.insert(tk.END, ' *', ("pinned",))
+
+        self.text.insert(tk.END, ' ' + utils.human_date(modifydate), ("modifydate",))
+
+        # tags can be None (newly created note) or [] or ['tag1', 'tag2']
+        if tags > 0:
+            if config.tagfound:
+                self.text.insert(tk.END, ' ' + ','.join(tags), ("found",))
+            else:
+                self.text.insert(tk.END, ' ' + ','.join(tags), ("tags",))
+
+
+        self.text.insert(tk.END, '\n')
+
+        self.disable_text()
+
+    def _bind_events(self):
+        # Text widget events ##########################################
+
+
+        self.text.bind("<Button 1>", self.cmd_text_button1)
+
+        # same deal as for pageup
+        # we have to stop the text widget class event handler from firing
+        def cmd_up(e):
+            self.select_prev(silent=False)
+            return "break"
+
+        self.text.bind("<Up>", cmd_up)
+
+        # for pageup, event handler needs to return "break" so that
+        # Text widget's default class handler for pageup does not trigger.
+        def cmd_pageup(e):
+            self.select_prev(silent=False, delta=10)
+            return "break"
+
+        self.text.bind("<Prior>", cmd_pageup)
+
+        def cmd_down(e):
+            self.select_next(silent=False)
+            return "break"
+
+        self.text.bind("<Down>", cmd_down)
+
+        def cmd_pagedown(e):
+            self.select_next(silent=False, delta=10)
+            return "break"
+
+        self.text.bind("<Next>", cmd_pagedown)
+
+
+    def cmd_text_button1(self, event):
+        # find line that was clicked on
+        text_index = self.text.index("@%d,%d" % (event.x, event.y))
+        # go from event coordinate to tkinter text INDEX to note idx!
+        idx = int(text_index.split('.')[0]) - 1
+        self.select(idx, silent=False)
+
+
+    def clear(self):
+        """
+
+        """
+        self.enable_text()
+        # clear everything from the display
+        self.text.delete(1.0, tk.END)
+        # and make sure our backing store is in sync
+        del self.note_headers[:]
+        self.disable_text()
+
+    def disable_text(self):
+        self.text.config(state=tk.DISABLED)
+
+    def enable_text(self):
+        self.text.config(state=tk.NORMAL)
+
+    def find_note_by_title(self, title):
+        """
+        Find note with given title.
+
+        @returns: Note index if found, -1 otherwise.
+        """
+
+        idx = -1
+        for i, nh in enumerate(self.note_headers):
+            t = nh[NotesList.TITLE_COL]
+            if t == title:
+                idx = i
+                break
+
+        return idx
+
+    def get_number_of_notes(self):
+        # could also have used:
+        # return int(self.text.index('end-1c').split('.')[0])
+        # but we have the backing store!
+        return len(self.note_headers)
+
+    def get_pinned(self, idx):
+        return self.note_headers[idx][NotesList.PINNED_COL]
+
+    def get_tags(self, idx):
+        """
+        @returns: raw list of tag strings, e.g. ['work', 'howto']
+        """
+        return self.note_headers[idx][NotesList.TAGS_COL]
+
+    def get_title(self, idx):
+        return self.note_headers[idx][NotesList.TITLE_COL]
+
+    def get_modifydate(self, idx):
+        """
+        Return modifydate of idx'th note.
+
+        @returns: modifydate as a floating point timestamp.
+        """
+        return self.note_headers[idx][NotesList.MODIFYDATE_COL]
+
+    def idx_to_index_range(self, idx):
+        """
+        Given a note index idx, return the Tkinter text index range for
+        the start and end of that note.
+        """
+
+        # tkinter text first line is 1, but first column is 0
+        row = idx+1
+        start = "%d.0" % (row,)
+        end = "%d.end" % (row,)
+
+        return (start, end)
+
+    def select(self, idx, silent=True):
+        """
+        @param idx: index of note to select. -1 if no selection.
+        """
+
+        # remove tag selected from row 1 (first) and column 0 to the end of the buffer
+        self.text.tag_remove("selected", "1.0", "end")
+
+        if idx >= 0 and idx < self.get_number_of_notes():
+            # then add it to the requested note line(s)
+            start, end = self.idx_to_index_range(idx)
+            self.text.tag_add("selected", start, end)
+            # ensure that this is visible
+            self.text.see(start)
+            # and store the current idx
+            self.selected_idx = idx
+
+        else:
+            self.selected_idx = -1
+
+        if not silent:
+            self.event_generate('<<NotesListSelect>>')
+
+    def select_next(self, silent=True, delta=1):
+        """
+        Select note right after the current selection.
+        """
+
+        new_idx = self.selected_idx + delta
+        if new_idx >= 0 and new_idx < self.get_number_of_notes():
+            self.select(new_idx, silent)
+
+        elif new_idx >= self.get_number_of_notes():
+            self.select(self.get_number_of_notes() - 1, silent)
+
+    def select_prev(self, silent=True, delta=1):
+        """
+        Select note right after the current selection.
+        """
+
+        new_idx = self.selected_idx - delta
+        if new_idx >= 0 and new_idx <= self.get_number_of_notes():
+            self.select(new_idx, silent)
+
+        elif new_idx < 0:
+            self.select(0, silent)
+
+tkinter_umlauts=['odiaeresis', 'adiaeresis', 'udiaeresis', 'Odiaeresis', 'Adiaeresis', 'Udiaeresis', 'ssharp']
+
+class TriggeredcompleteEntry(tk.Entry):
+    """
+    Subclass of tk.Entry that features triggeredcompletion.
+
+    To enable triggeredcompletion use set_completion_list(list) to define 
+    a list of possible strings to hit.
+    To cycle through hits use CTRL <space> keys.
+    """
+
+    def __init__(self, master, case_sensitive, **kw): 
+        tk.Entry.__init__(self, master, **kw)
+        self.case_sensitive = case_sensitive
+        self.bind('<KeyRelease>', self.handle_keyrelease)               
+
+    def set_completion_list(self, completion_list):
+        self._completion_list = completion_list
+        self._hits = []
+        self._hit_index = 0
+        self.position = 0               
+        self.cycle = 0
+
+    def triggeredcomplete(self):
+        """triggeredcomplete the Entry, delta may be 0/1 to cycle through possible hits"""
+        if self.cycle: # need to delete selection otherwise we would fix the current position
+            self.delete(self.position, tk.END)
+            self._hit_index += 1
+            if self._hit_index == len(self._hits):
+                self._hit_index = 0
+        else: # set position to end so selection starts where textentry ended
+            self.position = len(self.get())
+            # collect hits
+            _hits = []
+            for element in self._completion_list:
+                if self.case_sensitive == 0: 
+                    if element.lower().startswith(self.get().lower()):
+                         _hits.append(element)
+                else:
+                    if element.startswith(self.get()):
+                         _hits.append(element)
+            self._hit_index = 0
+            self._hits=_hits
+        # now finally perform the triggered completion
+        if self._hits:
+            self.delete(0,tk.END)
+            self.insert(0,self._hits[self._hit_index])
+            self.select_range(self.position,tk.END)
+
+    def handle_keyrelease(self, event):
+        """event handler for the keyrelease event on this widget"""
+        ctrl  = ((event.state & 0x0004) != 0)
+
+        if event.keysym == "BackSpace":
+            self.cycle = 0
+            self.delete(self.index(tk.INSERT), tk.END) 
+            self.position = self.index(tk.END)
+        if event.keysym == "Left":
+            self.cycle = 0
+            if self.position < self.index(tk.END): # delete the selection
+                self.delete(self.position, tk.END)
+            else:
+                self.position = self.position-1 # delete one character
+                self.delete(self.position, tk.END)
+        if event.keysym == "Right":
+            self.position = self.index(tk.END) # go to end (no selection)
+            self.cycle = 0
+        if event.keysym == "space" and ctrl:
+            # cycle 
+            self.triggeredcomplete()
+            if self.cycle == 0:
+                self.cycle = 1
+
+
 class View(utils.SubjectMixin):
     """Main user interface class.
     """
@@ -153,6 +487,7 @@ class View(utils.SubjectMixin):
         utils.SubjectMixin.__init__(self)
         
         self.config = config
+        self.taglist = []
         
         notes_list_model.add_observer('set:list', self.observer_notes_list)
         self.notes_list_model = notes_list_model
@@ -169,16 +504,16 @@ class View(utils.SubjectMixin):
         #self.user_text.focus_set()
 
         self.search_entry.focus_set()
-        
+
     def askyesno(self, title, msg):
         return tkMessageBox.askyesno(title, msg)
     
-    def cmd_lb_notes_select(self, evt):
-        sidx = self.get_selected_idx()
+    def cmd_notes_list_select(self, evt):
+        sidx = self.notes_list.selected_idx
         self.notify_observers('select:note', utils.KeyValueObject(sel=sidx))
         
     def cmd_root_delete(self, evt=None):
-        sidx = self.get_selected_idx()
+        sidx = self.notes_list.selected_idx
         self.notify_observers('delete:note', utils.KeyValueObject(sel=sidx))
         
     def cmd_root_new(self, evt=None):
@@ -195,13 +530,6 @@ class View(utils.SubjectMixin):
     def get_continuous_rendering(self):
         return self.continuous_rendering.get()
         
-    def get_selected_idx(self):
-        # no selection: s = ()
-        # something: s = ('idx',)
-        s = self.lb_notes.curselection()
-        sidx = int(s[0]) if s else -1
-        return sidx
-    
     def get_text(self):
         # err, you have to specify 1.0 to END, and NOT 0 to END like I thought.
         # also, see the comment by Bryan Oakley to
@@ -243,43 +571,15 @@ class View(utils.SubjectMixin):
         what you've just selected.
         """
 
-        self.lb_notes.select_clear(0, tk.END)
-        self.lb_notes.select_set(idx)
-        # we move the active (underlined) selection along, else we lose
-        # synchronization during arrow movements with the search entry selected
-        self.lb_notes.activate(idx)
-        
-        # make sure the selected index is visible
-        # this only moves the view if the selected note is really not visible
-        self.lb_notes.see(idx)
-        
-        if not silent:
-            # we have to generate event explicitly, it doesn't fire by itself in this case
-            self.lb_notes.event_generate('<<ListboxSelect>>')
-            
+        self.notes_list.select(idx, silent)
+
     def select_note_by_name(self, name):
-        note_names = self.lb_notes.get(0, 'end')
-        try:
-            idx = note_names.index(name)
-        except ValueError:
-            # name is not in the list
-            return -1
-        
-        else:
-            self.select_note(idx)
-            return idx
-        
-    def select_note_prev(self):
-        idx = self.get_selected_idx()
-        if idx > 0:
-            self.select_note(idx - 1)
-    
-    def select_note_next(self):
-        idx = self.get_selected_idx()
-        # self.lb_notes.index(tk.END) returns the number of items
-        if idx < self.lb_notes.index(tk.END) - 1:
-            self.select_note(idx + 1)
-            
+        idx = self.notes_list.find_note_by_title(name)
+        if idx >= 0:
+            self.select_note(idx, silent=False)
+
+        return idx
+
     def set_note_status(self, status):
         """status is an object with ivars modified, saved and synced.
         """
@@ -301,13 +601,15 @@ class View(utils.SubjectMixin):
     def _bind_events(self):
         # make sure window close also goes through our handler
         self.root.protocol('WM_DELETE_WINDOW', self.handler_close)
+
+        self.root.bind_all("<Control-g>", lambda e: self.tags_entry.focus())
         
-        self.lb_notes.bind("<<ListboxSelect>>", self.cmd_lb_notes_select)
+        self.notes_list.bind("<<NotesListSelect>>", self.cmd_notes_list_select)
         # same behaviour as when the user presses enter on search entry:
         # if something is selected, focus the text area
         # if nothing is selected, try to create new note with
         # search entry value as name
-        self.lb_notes.bind("<Return>", self.handler_search_enter)        
+        self.notes_list.text.bind("<Return>", self.handler_search_enter)
         
         self.search_entry.bind("<Escape>", lambda e:
                 self.search_entry.delete(0, tk.END))
@@ -316,18 +618,29 @@ class View(utils.SubjectMixin):
         self.search_entry.bind("<Return>", self.handler_search_enter)
         
         self.search_entry.bind("<Up>", lambda e:
-                               self.select_note_prev())
+            self.notes_list.select_prev(silent=False))
+        self.search_entry.bind("<Prior>", lambda e:
+            self.notes_list.select_prev(silent=False, delta=10))
+
         self.search_entry.bind("<Down>", lambda e:
-                               self.select_note_next())
-        
+            self.notes_list.select_next(silent=False))
+        self.search_entry.bind("<Next>", lambda e:
+            self.notes_list.select_next(silent=False, delta=10))
+
+
         self.text_note.bind("<<Change>>", self.handler_text_change)
         
         # user presses escape in text area, they go back to notes list
-        self.text_note.bind("<Escape>", lambda e: self.lb_notes.focus())
+        self.text_note.bind("<Escape>", lambda e: self.notes_list.text.focus())
         # <Key>
         
         self.text_note.bind("<Control-a>", self.cmd_select_all)
 
+        self.tags_entry_var.trace('w', self.handler_tags_entry)
+        self.tags_entry.bind("<Escape>", lambda e: self.text_note.focus())
+
+        self.pinned_checkbutton_var.trace('w', self.handler_pinned_checkbutton)
+
         self.root.after(self.config.housekeeping_interval_ms, self.handler_housekeeper)
 
     def _create_menu(self):
@@ -469,7 +782,9 @@ class View(utils.SubjectMixin):
         
         search_entry.make_style()
         self.search_entry_var = tk.StringVar()
-        self.search_entry = tk.Entry(search_frame, textvariable=self.search_entry_var, style="Search.entry")
+        #self.search_entry = tk.Entry(search_frame, textvariable=self.search_entry_var, style="Search.entry")
+        self.search_entry = TriggeredcompleteEntry(search_frame, self.config.case_sensitive, textvariable=self.search_entry_var, style="Search.entry")
+        #self.search_entry.set_completion_list(self.taglist)
         self.search_entry_var.trace('w', self.handler_search_entry)
         self.search_entry.pack(fill=tk.X,padx=5, pady=5)
         search_frame.pack(side=tk.TOP, fill=tk.X)
@@ -481,26 +796,31 @@ class View(utils.SubjectMixin):
         
         left_frame = tk.Frame(paned_window, width=100)
         paned_window.add(left_frame)
-       
-        # setup the scrollbar
-        self.sb_notes = tk.Scrollbar(left_frame, orient=tk.VERTICAL)
-        
-        # exportselection=0 means it doesn't automatically export to
-        # x selection. with that active, selecting in the text widget
-        # removes selection in listbox.
-        # thank you http://stackoverflow.com/a/756875
-        self.lb_notes = tk.Listbox(left_frame, exportselection=0,
-                                   yscrollcommand=self.sb_notes.set)
-        
-        self.sb_notes.config(command=self.lb_notes.yview)
-        self.sb_notes.pack(side=tk.RIGHT, fill=tk.Y)
-        
-        # need both fill and expand to make it fill all avail area
-        self.lb_notes.pack(fill=tk.BOTH, expand=1)
+
+        self.notes_list = NotesList(
+            left_frame,
+            self.config.list_font_family, self.config.list_font_size, self.config.background_color)
+        self.notes_list.pack(fill=tk.BOTH, expand=1)
 
         right_frame = tk.Frame(paned_window, width=400)
         paned_window.add(right_frame)
 
+        note_meta_frame = tk.Frame(right_frame)
+        note_meta_frame.pack(side=tk.BOTTOM, fill=tk.X)
+
+        pinned_label = tk.Label(note_meta_frame,text="Pinned")
+        pinned_label.pack(side=tk.LEFT)
+        self.pinned_checkbutton_var = tk.IntVar()
+        pinned_checkbutton = tk.Checkbutton(note_meta_frame, variable=self.pinned_checkbutton_var)
+        pinned_checkbutton.pack(side=tk.LEFT)
+
+        tags_label = tk.Label(note_meta_frame, text="Tags")
+        tags_label.pack(side=tk.LEFT)
+        self.tags_entry_var = tk.StringVar()
+        self.tags_entry = tk.Entry(note_meta_frame, textvariable=self.tags_entry_var)
+        self.tags_entry.pack(side=tk.LEFT, fill=tk.X, expand=1, pady=3, padx=3)
+
+
         # we'll use this method to create the different edit boxes
         def create_scrolled_text(master):
             yscrollbar = tk.Scrollbar(master)
@@ -514,7 +834,8 @@ class View(utils.SubjectMixin):
                                   wrap=tk.WORD,
                                   font=f, tabs=(4 * f.measure(0), 'left'), tabstyle='wordprocessor',
                                   yscrollcommand=yscrollbar.set,
-                                  undo=True)
+                                  undo=True,
+                                  background = self.config.background_color)
             # change default font at runtime with:
             text.config(font=f)
 
@@ -529,29 +850,6 @@ class View(utils.SubjectMixin):
         # setup user_text ###############################################
         self.text_note = create_scrolled_text(right_frame)
 
-#        def cb_ut_fi(event):
-#            self.set_current_text(CURTEXT_USER)
-#
-#        self.user_text.bind('<FocusIn>', cb_ut_fi)
-#
-#        def cb_ut_m(event):
-#            self.set_user_mode(MODE_MODIFIED)
-#
-#        self.user_text.bind('<<Change>>', cb_ut_m)
-#
-#        # setup sys_text ################################################
-#        self.sys_text, self._sys_mode_label_var = \
-#                      create_scrolled_text(bottom_frame, "System Environment")
-#
-#        def cb_st_fi(event):
-#            self.set_current_text(CURTEXT_SYS)
-#
-#        self.sys_text.bind('<FocusIn>', cb_st_fi)
-#
-#        def cb_st_c(event):
-#            self.set_sys_mode(MODE_MODIFIED)
-#            
-#        self.sys_text.bind('<<Change>>', cb_st_c)
 
         # finish UI creation ###########################################
 
@@ -560,8 +858,10 @@ class View(utils.SubjectMixin):
         
         # call update so we know that sizes are up to date
         self.root.update_idletasks()
-        
-        
+
+    def get_number_of_notes(self):
+        return self.notes_list.get_number_of_notes()
+
     def handler_close(self, evt=None):
         """Handler for exit menu command and close window event.
         """
@@ -612,46 +912,85 @@ class View(utils.SubjectMixin):
         refresh_notes_list = False
         prev_title = None
         prev_modifydate = None
+        prev_pinned = 0
         for i,o in enumerate(self.notes_list_model.list):
             # order should be the same as our listbox
             nt = utils.get_note_title(o.note)
-            ot = self.lb_notes.get(i)
+            ot = self.notes_list.get_title(i)
             # if we strike a note with an out-of-date title, redo.
             if nt != ot:
                 logging.debug('title "%s" resync' % (nt,))
                 refresh_notes_list = True
-                continue
-            
+                break
+
+            # compare modifydate timestamp in our notes_list_model to what's displayed
+            # if these are more than 60 seconds apart, we want to update our
+            # mod-date display.
+            md = float(o.note.get('modifydate', 0))
+            omd = self.notes_list.get_modifydate(i)
+            if abs(md - omd) > 60:
+                # we log the title
+                logging.debug('modifydate "%s" resync' % (nt,))
+                refresh_notes_list = True
+                break
+
+            pinned = utils.note_pinned(o.note)
+            old_pinned = self.notes_list.get_pinned(i)
+            if pinned != old_pinned:
+                # we log the title
+                logging.debug('pinned "%s" resync' % (nt,))
+                refresh_notes_list = True
+                break
+
+            tags = o.note.get('tags', 0)
+            old_tags = self.notes_list.get_tags(i)
+            if tags != old_tags:
+                # we log the title
+                logging.debug('tags "%s" resync' % (nt,))
+                refresh_notes_list = True
+                break
+
             if self.config.sort_mode == 0:
                 # alpha
                 if prev_title is not None and prev_title > nt:
                     logging.debug("alpha resort triggered")
                     refresh_notes_list = True
-                    continue
+                    break
                 
                 prev_title = nt
                 
             else:
-                md = float(o.note.get('modifydate', 0))
-                if prev_modifydate is not None and prev_modifydate < md:
+
+                # we go from top to bottom, newest to oldest
+                # this means that prev_modifydate (above) needs to be larger
+                # than md (below). if it's not, re-sort.
+                if prev_modifydate is not None and prev_modifydate < md and \
+                   not prev_pinned:
                     logging.debug("modifydate resort triggered")
                     refresh_notes_list = True
-                    continue
+                    break
                 
-                prev_modifydate = md 
+                prev_modifydate = md
+                if self.config.pinned_ontop:
+                    prev_pinned = utils.note_pinned(o.note)
             
         if refresh_notes_list:
             self.refresh_notes_list()
         
         self.root.after(self.config.housekeeping_interval_ms, self.handler_housekeeper)
         
+    def handler_pinned_checkbutton(self, *args):
+        self.notify_observers('change:pinned',
+            utils.KeyValueObject(value=self.pinned_checkbutton_var.get()))
+
     def handler_search_enter(self, evt):
         # user has pressed enter whilst searching
         # 1. if a note is selected, focus that
         # 2. if nothing is selected, create a new note with this title
 
-        if self.get_selected_idx() >= 0:
+        if self.notes_list.selected_idx >= 0:
             self.text_note.focus()
+            self.text_note.see(tk.INSERT)
             
         else:
             # nothing selected
@@ -663,6 +1002,10 @@ class View(utils.SubjectMixin):
         self.notify_observers('change:entry', 
                               utils.KeyValueObject(value=self.search_entry_var.get()))
 
+    def handler_tags_entry(self, *args):
+        self.notify_observers('change:tags',
+            utils.KeyValueObject(value=self.tags_entry_var.get()))
+
     def handler_click_link(self, link):
         if link.startswith('[['):
             link = link[2:-2]
@@ -686,7 +1029,10 @@ class View(utils.SubjectMixin):
         
         # take care of invalid regular expressions...
         try:
-            pat = re.compile(st)
+            if self.config.case_sensitive == 0: 
+                pat = re.compile(st, re.I)
+            else:
+                pat = re.compile(st)
         except re.error:
             return
         
@@ -768,11 +1114,16 @@ class View(utils.SubjectMixin):
             
     def main_loop(self):
         self.root.mainloop()
+
+    def mute_note_data_changes(self):
+        self.mute('change:text')
+        self.mute('change:tags')
+        self.mute('change:pinned')
         
     def set_status_text(self, txt):
         self.statusbar.set_status(txt)
         
-    def set_text(self, note_content, reset_undo=True):
+    def set_note_data(self, note, reset_undo=True):
         """Replace text in editor with content.
         
         This is usually called when a new note is selected (case 1), or
@@ -783,8 +1134,14 @@ class View(utils.SubjectMixin):
         """
         
         self.text_note.delete(1.0, tk.END) # clear all
-        self.text_note.insert(tk.END, note_content)
-        
+
+        if note is not None:
+            self.text_note.insert(tk.END, note['content'])
+            # default to an empty array for tags
+            tags=note.get('tags', [])
+            self.tags_entry_var.set(','.join(tags))
+            self.pinned_checkbutton_var.set(utils.note_pinned(note))
+
         if reset_undo:
             # usually when a new note is selected, we want to reset the
             # undo buffer, so that a user can't undo right into the previously
@@ -793,11 +1150,20 @@ class View(utils.SubjectMixin):
         
         
     def set_notes(self, notes):
-        # clear the listbox
-        self.lb_notes.delete(0, tk.END)
+        # clear the notes list
+        self.notes_list.clear()
+        taglist = []
         
         for o in notes:
-            self.lb_notes.insert(tk.END, utils.get_note_title(o.note))
+            tags = o.note.get('tags')
+            if tags:
+                taglist += tags
+            self.notes_list.append(o.note, utils.KeyValueObject(tagfound=o.tagfound))
+
+        taglist = list(set(self.taglist + taglist))
+        if len(taglist) > len(self.taglist):
+            self.taglist=taglist
+            self.search_entry.set_completion_list(self.taglist)
 
     def show_error(self, title, msg):
         tkMessageBox.showerror(title, msg)
@@ -808,13 +1174,27 @@ class View(utils.SubjectMixin):
     def show_warning(self, title, msg):
         tkMessageBox.showwarning(title, msg)
 
-    def update_selected_note_text(self, content):
+    def unmute_note_data_changes(self):
+        self.unmute('change:text')
+        self.unmute('change:tags')
+        self.unmute('change:pinned')
+
+
+    def update_selected_note_data(self, note):
+        """
+        Update currently selected note's data.
+
+        This is called only by the event handler for the per-note on-demand
+        syncing.
+        """
+
         # store cursor position
         cursor_pos = self.text_note.index(tk.INSERT)
-        self.mute('change:text')
-        self.set_text(content)
+        # the user is not changing anything, so we don't want the event to fire
+        self.mute_note_data_changes()
+        self.set_note_data(note)
         self.text_note.mark_set(tk.INSERT, cursor_pos)
-        self.unmute('change:text')
+        self.unmute_note_data_changes()
+
 
-       
         
